# --------------------------
# This script runs the analyses reported in:
# "Measuring the effectiveness of management interventions at regional scales
#   by integrating ecological monitoring and modelling"
# R. Freckleton et al. 
#



# ---- Load required packages ------ 
rm( list = ls() )

library( lattice )
library( VGAM )
library( MASS) 
library( lme4)
library( pbkrtest )
library( merTools)
library( ggplot2)

# - variable used to code levels
LVS <- c( 0, 1, 2, 3, 4 )

# ---- Load data -----


# Load data from focal farm
data <- read.csv( "VariableSowingExample.csv", h = T )
idx <- which( complete.cases(data) == TRUE )
data <- data[idx,]
variablefielddata <- subset(data, field == "variableSowingField" ) 
otherfieldsdata <- subset(data, field != "variableSowingField" )

variableN2014 <-  table( factor( variablefielddata$state2014 , levels = LVS ) ) 
variableN2015 <-  table( factor( variablefielddata$state2015 , levels = LVS ) ) 


# The subset of fields which rotated from Winter Wheat to Spring Barley
fulldataWWSB <- read.csv( "WW_SBsubset.csv", h = T)
idx <- which( complete.cases(fulldataWWSB) == TRUE )
fulldataWWSB <- fulldataWWSB[idx,]
fulldataWWSB <- rbind( fulldataWWSB, otherfieldsdata)
fulldataWWSB$field <- droplevels( fulldataWWSB$field )
fulldataWWSB <- fulldataWWSB[ order(fulldataWWSB$field), ]


# The subset of fields which rotated from Winter Wheat to Winter Wheat
fulldataWWWW <- read.csv( "WW_WWsubset.csv", h = T)
idx <- which( complete.cases(fulldataWWWW) == TRUE )
fulldataWWWW <- fulldataWWWW[idx,]
fulldataWWWW$field <- droplevels( fulldataWWWW$field )
fulldataWWWW <- fulldataWWWW[ order(fulldataWWWW$field), ]
# ------------------------------------------------------------------------





# ------ Some basic data plots --------- 
# Reported in Figure 1

summaryWWSB2014 <- table( as.factor( fulldataWWSB$state2014), fulldataWWSB$field )
summaryWWSB2015 <- table( as.factor( fulldataWWSB$state2015), fulldataWWSB$field )

summaryWWSB2014 <- apply( summaryWWSB2014, 2, function(x) x / sum(x) )
summaryWWSB2015 <- apply( summaryWWSB2015, 2, function(x) x / sum(x) )

summaryWWWW2014 <- table( as.factor( fulldataWWWW$state2014), fulldataWWWW$field )
summaryWWWW2015 <- table( as.factor( fulldataWWWW$state2015), fulldataWWWW$field )

summaryWWWW2014 <- apply( summaryWWWW2014, 2, function(x) x / sum(x) )
summaryWWWW2015 <- apply( summaryWWWW2015, 2, function(x) x / sum(x) )

layout( matrix(c(1,2,3,4), ncol = 2, byrow = TRUE) )
# rob.par™ - my trademark plotting parameters
rob.par <- par(bty = "l",
		cex.lab = 1.1,
		tcl = 0.3,
		las = 1,
		pch = 1,
		bty = "l")

plot( LVS, summaryWWSB2014[,1] , type = "l", col = "grey", ylim = c(0,1), ylab = "", xlab = "", xaxt = "n" )
apply( summaryWWSB2014[,2:12],2, function(y) lines( LVS, y, col = "grey", ylim = c(0,1), ylab = "", xlab = "", xaxt = "n" ) )
mn <- apply( summaryWWSB2014, 1, mean) 
points( LVS, mn, col = "black", pch = 19)

plot( LVS, summaryWWSB2015[,1] , type = "l", col = "grey", ylim = c(0,1), ylab = "", xlab = "", xaxt = "n" )
apply( summaryWWSB2015[,2:12],2, function(y) lines( LVS, y, col = "grey", ylim = c(0,1), ylab = "", xlab = "", xaxt = "n" ) )
mn <- apply( summaryWWSB2015, 1, mean) 
points( LVS, mn, col = "black", pch = 19)

plot( LVS, summaryWWWW2014[,1] , type = "l", col = "grey", ylim = c(0,1), ylab = "", xlab = "", xaxt = "n" )
apply( summaryWWWW2014[,2:22],2, function(y) lines( LVS, y, col = "grey", ylim = c(0,1), ylab = "", xlab = "", xaxt = "n" ) )
mn <- apply( summaryWWWW2014, 1, mean) 
points( LVS, mn, col = "black", pch = 19)

plot( LVS, summaryWWWW2015[,1] , type = "l", col = "grey", ylim = c(0,1), ylab = "", xlab = "", xaxt = "n" )
apply( summaryWWWW2015[,2:22],2, function(y) lines( LVS, y, col = "grey", ylim = c(0,1), ylab = "", xlab = "", xaxt = "n" ) )
mn <- apply( summaryWWWW2015, 1, mean) 
points( LVS, mn, col = "black", pch = 19)
# ------------------------------------------------------------------------


# --- First set of stats ----- 
# --- compare 2014 densities between WWWW and WWSB 

mnWWWW14 <- c(0:4) %*% summaryWWWW2014 
mnWWSB14 <- c(0:4) %*% summaryWWSB2014
mnWWWW15 <- c(0:4) %*% summaryWWWW2015
mnWWSB15 <- c(0:4) %*% summaryWWSB2015

t.test( mnWWWW14, mnWWSB14)
t.test( mnWWWW15, mnWWSB15)



t.test( mnWWWW14 - mnWWWW15)
t.test( mnWWSB14 - mnWWSB15 )

y1 <- mnWWWW15 - mnWWWW14
y2 <- mnWWSB15 - mnWWSB14

t.test( y1, y2)

densityDif <- c(y1, y2)
crop <- c(rep("WWWW", length( y1 ) ) , rep( "WWSB", length(y2) )  )
summary( lm( densityDif ~ crop -1) )



# -------- Fit VGLM models with multinomial errors ----------- 
modfocalfield <- vglm( state2015 ~ as.factor( state2014 ), data = variablefielddata, family = multinomial )
modWWSB0 <- vglm( state2015 ~ 1, data = fulldataWWSB, family = multinomial(refLevel = "0"  ) )
modWWSB1 <- vglm( state2015 ~ as.factor( state2014 ) -1 , data = fulldataWWSB, family = multinomial(refLevel = "0"  )  )
modWWSB2 <- vglm( state2015 ~ field + as.factor( state2014 ) -1, data = fulldataWWSB, family = multinomial(refLevel = "0"  )  )

modWWWW0 <- vglm( state2015 ~ 1, data = fulldataWWWW, family = multinomial  )
modWWWW1 <- vglm( state2015 ~  as.factor( state2014 ) - 1 , data = fulldataWWWW, family = multinomial  )
modWWWW2 <- vglm( state2015 ~ field + as.factor( state2014 ) - 1 , data = fulldataWWWW, family = multinomial  )
# ------------------------------------------------------------------------




# --------- Use fitted models to generate field-scale matrices ---------
fieldsWWSB <- levels( fulldataWWSB$field )
nfWWSB <- length(fieldsWWSB)
newdataSB <- data.frame( state2014 = rep( c(0:4), nfWWSB), field = sort( rep(fieldsWWSB, 5) )  )
predWWSB <- predict( modWWSB2, newdataSB , type = "response")
# predWWSB.brm <- predict( modWWSB2.brm, newdataSB , type = "response")

fieldsWWWW <- levels( fulldataWWWW$field )
nfWWWW <- length(fieldsWWWW)
newdataWW <- data.frame( state2014 = rep( c(0:4), nfWWWW), field = sort( rep(fieldsWWWW, 5) )  )
predWWWW <- predict( modWWWW2, newdataWW , type = "response")
# ------------------------------------------------------------------------





# ---------- Function to extract matrices from predicted  ------------
extractFun <- function(i, pred) {
	idxstart <- (i-1) * 5 + 1
	idxfinish <- (i-1) * 5 + 5
	mat <- pred[ c(idxstart:idxfinish), 1:5 ]
	return( t( mat) )
	}

matsWWSB <- lapply( c(1:nfWWSB), extractFun, pred = predWWSB  )
ssdsWWSB <- lapply(matsWWSB, function(m) m %*% N2014  )

matsWWWW <- lapply( c(1:nfWWWW), extractFun, pred = predWWWW  )
ssdsWWWW <- lapply(matsWWWW, function(m) m %*% N2014  )
# ------------------------------------------------------------------------



# ------ predictions based on common start, i.e. densities in fields that were WW -> SB --------
# Repoted in Figure 2
# rob.par™ - my trademark plotting parameters
rob.par <- par(bty = "l",
		cex.lab = 1.1,
		tcl = 0.3,
		las = 1,
		pch = 1,
		bty = "l")
		
N2014 <- table( factor( fulldataWWWW$state2014 , levels = LVS ) ) 

ssdsWWSB <- lapply(matsWWSB, function(m) m %*% N2014  )
ssdsWWWW <- lapply(matsWWWW, function(m) m %*% N2014  )
		
layout( matrix( c(1,2), ncol  = 2 ) )

plot(variableN2015 / sum(variableN2015) , ylim = c(0, 1) , col = "red", ylab = "", xaxt = "n", type = "l")
lapply(ssdsWWSB, function(p) lines( c(0:4), p /sum(p) , lty = "22", col = "grey") )
ssdsWWSB <- matrix( unlist(ssdsWWSB), ncol = 5, byrow = TRUE)
avWWSB <- apply(ssdsWWSB, 2, mean)
points( c(0:4), avWWSB / sum(avWWSB ), pch = 19, col = "black")
lines(LVS, N2014 / sum(N2014), col = "blue", lty = "22")


ssdsWWWWul <- matrix( unlist(ssdsWWWW), ncol = 5, byrow = TRUE)
avWWWW <- apply(ssdsWWWWul, 2, mean)
plot( c(0:4), avWWWW / sum( avWWWW) , ylim = c(0, 1), pch = 19, col = "black", xlab = "", ylab = "", xaxt = "n" )
lines(LVS, N2014 / sum(N2014), col = "blue", lty = "22")
lapply(ssdsWWWW, function(p) lines( c(0:4), p / sum(p) , lty = "22", col = "grey") )
# ------------------------------------------------------------------------




# ------ predictions based on worse case start, i.e. fields that were WW -> WW --------
# Reported in Figure 3
# rob.par™ - my trademark plotting parameters
rob.par <- par(bty = "l",
		cex.lab = 1.1,
		tcl = 0.3,
		las = 1,
		pch = 1,
		bty = "l")

N2014 <- table( factor( fulldataWWSB$state2014 , levels = LVS ) ) 
N2015 <- table( factor( fulldataWWSB$state2015 , levels = LVS ) ) 
ssdsWWSB <- lapply(matsWWSB, function(m) m %*% N2014  )
ssdsWWWW <- lapply(matsWWWW, function(m) m %*% N2014  )
		
layout( matrix( c(1,2), ncol  = 2 ) )

plot(variableN2015 / sum(variableN2015), ylim = c(0, 1) , col = "red", ylab = "", xaxt = "n", type = "l")
lapply(ssdsWWSB , function(p) lines( c(0:4), p/sum(p)  , lty = "22", col = "grey") )
ssdsWWSB <- matrix( unlist(ssdsWWSB), ncol = 5, byrow = TRUE)
avWWSB <- apply(ssdsWWSB, 2, mean)
points( c(0:4), avWWSB / sum( avWWSB ) , pch = 19, col = "black")
lines(LVS, N2014 / sum(N2014), col = "blue", lty = "22")


ssdsWWWWul <- matrix( unlist(ssdsWWWW), ncol = 5, byrow = TRUE)
avWWWW <- apply(ssdsWWWWul, 2, mean)
plot( c(0:4), avWWWW / sum( avWWWW) , ylim = c(0,1), pch = 19, col = "black", xlab = "", ylab = "", xaxt = "n" )
lines(LVS, N2014 / sum(N2014) , col = "blue", lty = "22")
lapply(ssdsWWWW, function(p) lines( c(0:4), p /sum(p)  , lty = "22", col = "grey") )
# ------------------------------------------------------------------------





# ------ Full factorial combinations of matrices x intitial conditions x rotation --------- 
# Reported in Figure 4

# Simple summary function
count <- function(data, field, levels = LVS){
	idx <- which( data$field == field)
	subdat <- data[idx,]
	table( factor( subdat$state2014 , levels = LVS ) )
	}

N2014WWWWfields <- levels( fulldataWWWW$field )
N2014WWSBfields <- levels( fulldataWWSB$field )
N2014WWWWinits <- sapply( N2014WWWWfields, function(f) count( fulldataWWWW, f) )
N2014WWSBinits <- sapply( N2014WWSBfields, function(f) count( fulldataWWSB, f) )



iterateTWWWW <- function(T) apply( N2014WWWWinits,2, function(N) sum( LVS * T %*% N ) / sum(N) )
ssdsWWWW <- sapply( matsWWWW, iterateTWWWW) 

iterateTWWSB <- function(T) apply( N2014WWSBinits,2, function(N) sum( LVS * T %*% N ) / sum(N) )
ssdsWWSB <- sapply( matsWWSB, iterateTWWSB)

ssdsWWWW.WWSB <- sapply( matsWWSB, iterateTWWWW) 
ssdsWWSB.WWWW <- sapply( matsWWWW, iterateTWWSB)




dataWWWW <- data.frame( density = matrix( ssdsWWWW , ncol  = 1) )
dataWWWW$field <- rep( rownames( ssdsWWWW), 22)
dataWWWW$matrixname <- sort( rep( rownames( ssdsWWWW), 22) )
dataWWWW$rotation <- rep( "WWWW", 22*22)
dataWWWW$actualrot <- rep( "WWWW", 22*22 )
dataWWWW$initrot <- c( c(0:4) %*%N2014WWWWinits / colSums(N2014WWWWinits ) )


dataWWSB <- data.frame( density = matrix( ssdsWWSB , ncol  = 1) )
dataWWSB$field <- rep( rownames( ssdsWWSB), 12)
dataWWSB$matrixname <- sort( rep( rownames( ssdsWWSB), 12) )
dataWWSB$rotation <- rep( "WWSB", 12*12)
dataWWSB$actualrot <- rep( "WWSB", 12*12 )
dataWWSB$initrot <- c( c(0:4) %*%N2014WWSBinits / colSums(N2014WWSBinits ))

dataWWWW.WWSB <- data.frame( density = matrix( ssdsWWWW.WWSB, ncol  = 1) )
dataWWWW.WWSB$field <- rep( rownames( ssdsWWWW.WWSB), 12)
dataWWWW.WWSB$matrixname <- sort( rep( rownames( ssdsWWWW), 12) )
dataWWWW.WWSB$rotation <- rep( "WWWW", 22*12)
dataWWWW.WWSB$actualrot <- rep( "WWSB", 22*12 )
dataWWWW.WWSB$initrot <-c( c(0:4) %*%N2014WWWWinits / colSums(N2014WWWWinits ) )


dataWWSB.WWWW <- data.frame( density = matrix( ssdsWWSB.WWWW, ncol  = 1) )
dataWWSB.WWWW$field <- rep( rownames( ssdsWWSB.WWWW), 22)
dataWWSB.WWWW$matrixname <- sort( rep( rownames( ssdsWWSB), 22) )
dataWWSB.WWWW$rotation <- rep( "WWSB", 22*12)
dataWWSB.WWWW$actualrot <- rep( "WWWW", 22*12 )
dataWWSB.WWWW$initrot <-  c( c(0:4) %*%N2014WWSBinits / colSums(N2014WWSBinits ) )

summaryData <- rbind( dataWWWW, dataWWSB, dataWWWW.WWSB, dataWWSB.WWWW )


# Build up models describing change in density as a function of rotation, origin and initial states
# Use model2.b to summarise full analysis
model0 <-  lm(  (density  -  initrot ) ~ rotation , data = summaryData)
model1 <-  lm(  (density  -  initrot ) ~ rotation + field%in%rotation + matrixname%in%rotation , data = summaryData)

model2 <-  lmer(  (density  -  initrot ) ~ rotation + (1|field) + (1|matrixname), data = summaryData, REML = FALSE)
model2.b <-  lmer(  (density  -  initrot ) ~ 1 + (1|field) + (1|matrixname) + (1|rotation), data = summaryData, REML = FALSE)


re <- REsim( model2.b)
plotREsim( re )


